/** Rill v0.0.550 https://www.npmjs.com/package/rill License: MIT*/
var browser, byteLength, htmlReg, isType, statuses;

isType = require("@rill/is-type");

{
  browser = require("./browser");
}

htmlReg = /^\s*</;

statuses = {
  empty: {
    204: true,
    205: true,
    304: true
  },
  redirect: {
    300: true,
    301: true,
    302: true,
    303: true,
    305: true,
    307: true,
    308: true
  }
};


/*
 * Calculate bytelengths for complex strings.
#
 * @param {String} str
 * @return {Number}
 */

byteLength = function(str) {
  var char, code, i, len, len1;
  if (!str) {
    return 0;
  }
  str = str.toString();
  len = str.length;
  for (i = 0, len1 = str.length; i < len1; i++) {
    char = str[i];
    code = char.charCodeAt();
    if ((0x7f < code && code <= 0x7ff)) {
      len += 1;
    } else if ((0x7ff < code && code <= 0xffff)) {
      len += 2;
    } else if ((0xDC00 <= code && code <= 0xDFFF)) {
      len -= 1;
    }
  }
  return len;
};


/*
 * A default middleware that handles sending the response for the request.
#
 * @param {Object} req
 * @param {Object} res
 * @param {Function} next
 */

module.exports = regeneratorRuntime.mark(function callee$0$0(req, res, next) {
  var body, cookie, headers, method, originalRes, redirectURL, status;

  return regeneratorRuntime.wrap(function callee$0$0$(context$1$0) {
    while (1) switch (context$1$0.prev = context$1$0.next) {
    case 0:
      context$1$0.next = 2;
      return next;
    case 2:
      if (!res.isSent()) {
        context$1$0.next = 4;
        break;
      }

      return context$1$0.abrupt("return");
    case 4:
      method = req.method, redirectURL = req.redirectURL;
      originalRes = res.res, headers = res.headers, status = res.status, body = res.body;

      if (!(redirectURL != null)) {
        context$1$0.next = 10;
        break;
      }

      if (headers["location"] == null) {
        headers["location"] = redirectURL;
      }
      if (!statuses.redirect[status]) {
        status = 302;
      }
      return context$1$0.abrupt("return", browser.goTo(redirectURL));
    case 10:
      cookie = req.cookies.serialize();
      if (cookie != null) {
        headers["set-cookie"] = cookie;
        {
          window.document.cookie = cookie;
        }
      }
      if (Number(status) === 404 && (body != null)) {
        status = 200;
      }

      if (!(statuses.empty[status] || body == null)) {
        context$1$0.next = 18;
        break;
      }

      if (headers["content-type"] == null) {
        headers["content-type"] = void 0;
      }
      body = void 0;
      context$1$0.next = 34;
      break;
    case 18:
      if (!isType.String(body)) {
        context$1$0.next = 22;
        break;
      }

      if (headers["content-type"] == null) {
        headers["content-type"] = "text/" + (htmlReg.test(body) ? "html" : "plain") + "; charset=UTF-8";
      }
      context$1$0.next = 34;
      break;
    case 22:
      if (!isType.Buffer(body)) {
        context$1$0.next = 26;
        break;
      }

      headers["content-length"] = body.length;
      context$1$0.next = 34;
      break;
    case 26:
      if (!isType.Stream(body)) {
        context$1$0.next = 32;
        break;
      }

      if (headers["content-type"] == null) {
        headers["content-type"] = "application/octet-stream";
      }
      originalRes.writeHead(status, headers);
      return context$1$0.abrupt("return");
    case 32:
      if (headers["content-type"] == null) {
        headers["content-type"] = "application/json; charset=UTF-8";
      }
      try {
        body = JSON.stringify(body);
      } catch (_error) {}
    case 34:
      if (headers["content-length"] == null) {
        headers["content-length"] = byteLength(body);
      }
    case 35:
    case "end":
      return context$1$0.stop();
    }
  }, callee$0$0, this);
});
