event     = require("component-event")
serialize = require("@rill/form-json")
parseURL  = require("@rill/parse-url")
expect    = require("../util/expect")
referrer  = window.document.referrer

# Store registered createServer callback.
createRequest = ->

browser =
	###
	# Listen to all url change events on a dom element and trigger the server callback.
	#
	# @param {DOMNode} element
	# @api private
	###
	listen: (element = window)->
		event.bind(element, "DOMContentLoaded", handleURLChange)
		event.bind(element, "popstate", handleURLChange)
		event.bind(element, "submit", handleForm)
		event.bind(element, "click", handleLink)
		return browser

	###
	# Add a handler to listen to url changes.
	#
	# @param {Function} handler
	# @api private
	###
	createServer: (handler)->
		createRequest = handler
		return browser

	###
	# Trigger the registered handler to navigate to a given url.
	#
	# @param {String} url
	# @param {Object} options
	# @api private
	###
	goTo: (url, { method, headers, body, files, title } = {})->
		title   ?= ""
		headers ?= {}
		headers.referrer = referrer

		req = createRequest(
			url:     url
			body:    body
			files:   files
			method:  method ? "GET"
			headers: headers
		)

		req.then(({ redirected } = {})->
			unless redirected
				referrer = url
				window.history.pushState(null, title, url)
			return
		)

###
# Handle an event that changed the url (popstate or page load)
#
# @param {Object} event
###
handleURLChange = (event)->
	browser.goTo(window.location.href,
		method: "GET",
		replaceState: true
	)

###
# Handle intercepting link clicks to update the url.
#
# @param {Object} event
###
handleLink = (event)->
	# Ignore canceled events, modified clicks, and right clicks.
	return if event.defaultPrevented or
			  event.metaKey  or
			  event.ctrlKey  or
			  event.shiftKey or
			  event.button   isnt 0

	# Get the <a> element.
	el = event.target
	el = el.parentNode while el? and el.nodeName isnt "A"

	# Ignore clicks from non-a elements.
	return unless el?

	# Ignore the click if the element has a target.
	return if el.target and el.target isnt "_self"

	# Ignore the click if it's a download link. (We use this method of
	# detecting the presence of the attribute for old IE versions.)
	return if Boolean(el.attributes.download)

	# Use a url parser to parse URLs instead of relying on the browser
	# to do it for us (because IE).
	url = el.href
	{ protocol, host } = parseURL(url, window.location)

	# Ignore links that don't share a protocol and host with ours.
	return unless window.location.protocol.indexOf(protocol) isnt -1 and
				  window.location.host is host

	# Ignore 'rel="external"' links.
	return if el.rel and /(?:^|\s+)external(?:\s+|$)/.test(el.rel)

	browser.goTo(url)
	event.preventDefault()

###
# Handle intercepting forms to update the url.
#
# @param {Object} event
###
handleForm = (event)->
	# Ignore canceled events, modified clicks, and right clicks.
	return if event.defaultPrevented or
			  event.metaKey  or
			  event.ctrlKey  or
			  event.shiftKey

	# Get the <form> element.
	el = event.target
	el = el.parentNode while el? and el.nodeName isnt "FORM"

	# Ignore clicks from non-form elements.
	return unless el?

	# Ignore the click if the element has a target.
	return if el.target and el.target isnt "_self"

	# Use a url parser to parse URLs instead of relying on the browser
	# to do it for us (because IE).
	url = el.action
	{ protocol, host } = parseURL(url, window.location)

	# Ignore links that don't share a protocol and host with ours.
	return unless window.location.protocol.indexOf(protocol) isnt -1 and
				  window.location.host is host

	{ body, files } = serialize(el)
	browser.goTo(url,
		method: el.method
		body:   body
		files:  files
	)
	event.preventDefault()

module.exports = browser
